import React from "react";
import { createPortal } from "react-dom";
function isPromise(obj) {
    return (!!obj &&
        (typeof obj === "object" || typeof obj === "function") &&
        typeof obj.then === "function");
}
export const version = "%VERSION%";
export class Portal extends React.Component {
    constructor() {
        super(...arguments);
        this.unmount = false;
        this.mounted = false;
        this.seqId = 1;
        this.state = {
            container: null,
        };
    }
    static getDerivedStateFromProps(nextProps, nextState) {
        return {
            container: nextProps.container && isPromise(nextProps.container)
                ? nextState.container
                : nextProps.container,
        };
    }
    componentDidUpdate() {
        this.seqId++;
        if (isPromise(this.props.container)) {
            const currentSeqId = this.seqId;
            this.props.container.then(newContainer => {
                if (this.unmount ||
                    this.state.container === newContainer ||
                    this.seqId !== currentSeqId) {
                    return;
                }
                if (!this.mounted && newContainer && this.props.onChildrenMount) {
                    this.mounted = true;
                    this.props.onChildrenMount();
                }
                this.setState({
                    container: newContainer,
                });
            });
        }
        else if (this.props.container) {
            if (!this.mounted && this.props.onChildrenMount) {
                this.mounted = true;
                this.props.onChildrenMount();
            }
        }
    }
    componentDidMount() {
        if (this.props.container &&
            !isPromise(this.props.container) &&
            this.props.onChildrenMount) {
            this.mounted = true;
            this.props.onChildrenMount();
        }
        this.componentDidUpdate();
    }
    componentWillUnmount() {
        this.unmount = true;
    }
    render() {
        const container = this.state.container;
        if (!container)
            return null;
        return createPortal(this.props.children, container);
    }
}
Portal.defaultProps = {
    container: typeof document !== "undefined" ? document.body : null,
};
export default Portal;
